Imports System.Text
Imports Microsoft.CodeAnalysis
Imports Microsoft.CodeAnalysis.CSharp
Imports Microsoft.CodeAnalysis.CSharp.Symbols
Imports Microsoft.CodeAnalysis.CSharp.Syntax
Imports Microsoft.CodeAnalysis.Text
Imports Microsoft.CodeAnalysis.Editing

Namespace LanguageAgnosticCodeGeneration
	Friend Class Program
		Shared Sub Main(ByVal args() As String)

			' Roslyn code generation sample 

			' Get a workspace
			Dim workspace = New AdhocWorkspace()

			' Get the SyntaxGenerator for the specified language
			Dim generator = SyntaxGenerator.GetGenerator(workspace, LanguageNames.CSharp)

			' Create using/Imports directives
			Dim usingDirectives = generator.NamespaceImportDeclaration("System")

			' Generate two private fields
			Dim lastNameField = generator.FieldDeclaration("_lastName", generator.TypeExpression(SpecialType.System_String), Accessibility.Private)
			Dim firstNameField = generator.FieldDeclaration("_firstName", generator.TypeExpression(SpecialType.System_String), Accessibility.Private)

			' Generate two properties with explicit get/set 
			Dim lastNameProperty = generator.PropertyDeclaration("LastName", generator.TypeExpression(SpecialType.System_String), Accessibility.Public, getAccessorStatements:= New SyntaxNode() { generator.ReturnStatement(generator.IdentifierName("_lastName")) }, setAccessorStatements:= New SyntaxNode() { generator.AssignmentStatement(generator.IdentifierName("_lastName"), generator.IdentifierName("value"))})

			Dim firstNameProperty = generator.PropertyDeclaration("FirstName", generator.TypeExpression(SpecialType.System_String), Accessibility.Public, getAccessorStatements:= New SyntaxNode() { generator.ReturnStatement(generator.IdentifierName("_firstName")) }, setAccessorStatements:= New SyntaxNode() { generator.AssignmentStatement(generator.IdentifierName("_firstName"), generator.IdentifierName("value")) })

			' Generate the method body for the Clone method
			Dim cloneMethodBody = generator.ReturnStatement(generator.InvocationExpression(generator.IdentifierName("MemberwiseClone")))

			' Generate the Clone method declaration
			Dim cloneMethoDeclaration = generator.MethodDeclaration("Clone", Nothing, Nothing, Nothing, Accessibility.Public, DeclarationModifiers.Virtual, New SyntaxNode() { cloneMethodBody })

			' Generate a SyntaxNode for the interface's name you want to implement
			Dim ICloneableInterfaceType = generator.IdentifierName("ICloneable")

			' Explicit ICloneable.Clone implementation
			Dim cloneMethodWithInterfaceType = generator.AsPublicInterfaceImplementation(cloneMethoDeclaration, ICloneableInterfaceType)

			' Generate parameters for the class' constructor
			Dim constructorParameters = New SyntaxNode() { generator.ParameterDeclaration("LastName", generator.TypeExpression(SpecialType.System_String)), generator.ParameterDeclaration("FirstName", generator.TypeExpression(SpecialType.System_String)) }

			' Generate the constructor's method body
			Dim constructorBody = New SyntaxNode() { generator.AssignmentStatement(generator.IdentifierName("_lastName"), generator.IdentifierName("LastName")), generator.AssignmentStatement(generator.IdentifierName("_firstName"), generator.IdentifierName("FirstName"))}

			' Generate the class' constructor
			Dim constructor = generator.ConstructorDeclaration("Person", constructorParameters, Accessibility.Public, statements:= constructorBody)

			' An array of SyntaxNode as the class members
			Dim members = New SyntaxNode() { lastNameField, firstNameField, lastNameProperty, firstNameProperty, cloneMethodWithInterfaceType, constructor }

			' Generate the class
			Dim classDefinition = generator.ClassDeclaration("Person", typeParameters:= Nothing, accessibility:= Accessibility.Public, modifiers:= DeclarationModifiers.Abstract, baseType:= Nothing, interfaceTypes:= New SyntaxNode() { ICloneableInterfaceType }, members:= members)

			' Declare a namespace
			Dim namespaceDeclaration = generator.NamespaceDeclaration("MyTypes", classDefinition)

			' Get a CompilationUnit (code file) for the generated code
			Dim newNode = generator.CompilationUnit(usingDirectives, namespaceDeclaration).NormalizeWhitespace()

			Console.WriteLine(newNode.ToString())

			Console.Read()
			workspace.Dispose()
		End Sub
	End Class
End Namespace
